% Calculate F from P (& global transition matrices F_Ni, F_dz1, F_dz2)
function F = updateF(P)
global I J M T NS NT F_Ni F_dz1 F_dz2;

F_Nj = ones(I,J,NS,NS,NT);

j = 1;  % F_Nj does not vary with own choice (j = 1:J), so just fill 'j = 1' case & copy

% For a given combination of (Ni, dz1, dz2, mtype), make the state indexes (x, x0, x1, x2, x3)

for mtype = 1:NT
    
    % Beliefs: mapping from today's state (Ni,Nj,dz1,dz2) to tomorrow's Nj = {0,1,2,3}
    fnj_mcd = zeros(NS,4);      % McDonald's belief
    fnj_other = zeros(NS,4);    % Other 4 chains beliefs
    
    for ni = 0:3
        for z1 = 0:3
            for z2 = 0:3
                
                nj = 0;         % If today's # of rivals = 0
                x = 1 + (4*4*4)*ni + (4*4)*nj + 4*z1 + z2;  % index of today's own state
                x0 = 1 + (4*4*4)*0 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 0 shops
                x1 = 1 + (4*4*4)*1 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 1 shops
                x2 = 1 + (4*4*4)*2 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 2 shops
                x3 = 1 + (4*4*4)*3 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 3 shops
                
                fnj_mcd(x,1) = P(2,2,x0,mtype)^4;                               % Prob(Nj'= 0) from McD's perspective
                fnj_mcd(x,2) = 4 * P(2,3,x0,mtype) * (P(2,2,x0,mtype)^3);       % Prob(Nj'= 1) from McD's perspective
                fnj_mcd(x,3) = 6 * (P(2,3,x0,mtype)^2) * (P(2,2,x0,mtype)^2);   % Prob(Nj'= 2) from McD's perspective
                fnj_mcd(x,4) = 4 * (P(2,3,x0,mtype)^3) * P(2,2,x0,mtype);       % Prob(Nj'= 3) from McD's perspective
                
                fnj_other(x,1) = P(1,2,x0,mtype) * (P(2,2,x0,mtype)^3);         % Prob(Nj'= 0) etc. from Others' perspectives
                fnj_other(x,2) = P(1,3,x0,mtype) * (P(2,2,x0,mtype)^3)...       
                    + P(1,2,x0,mtype) * 3 * P(2,3,x0,mtype) * (P(2,2,x0,mtype)^2);  
                fnj_other(x,3) = P(1,3,x0,mtype) * 3 * P(2,3,x0,mtype) * (P(2,2,x0,mtype)^2)...
                    + P(1,2,x0,mtype) * 3 * (P(2,3,x0,mtype)^2) * P(2,2,x0,mtype);
                fnj_other(x,4) = P(1,3,x0,mtype) * 3 * (P(2,3,x0,mtype)^2) * P(2,2,x0,mtype)...
                    + P(1,2,x0,mtype) * (P(2,3,x0,mtype)^3);
                
                nj = 1;         % If today's # of rivals = 1                               
                x = 1 + (4*4*4)*ni + (4*4)*nj + 4*z1 + z2;  % index of today's own state
                x0 = 1 + (4*4*4)*0 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 0 shops
                x1 = 1 + (4*4*4)*1 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 1 shops
                x2 = 1 + (4*4*4)*2 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 2 shops
                x3 = 1 + (4*4*4)*3 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 3 shops
                
                fnj_mcd(x,1) = P(2,1,x1,mtype) * (P(2,2,x0,mtype)^3);
                fnj_mcd(x,2) = P(2,2,x1,mtype) * (P(2,2,x0,mtype)^3);
                fnj_mcd(x,3) = P(2,3,x1,mtype) * (P(2,2,x0,mtype)^3)...
                    + P(2,2,x1,mtype) * 3 * P(2,3,x0,mtype) * (P(2,2,x0,mtype)^2);
                fnj_mcd(x,4) = P(2,3,x1,mtype) * 3 * P(2,3,x0,mtype) * (P(2,2,x0,mtype)^2)...
                    + P(2,2,x1,mtype) * 3 * (P(2,3,x0,mtype)^2) * P(2,2,x0,mtype);
                
                fnj_other(x,1) = .5 * (P(1,1,x1,mtype) * P(2,2,x0,mtype)^3)...
                    + .5 * (P(1,2,x0,mtype) * P(2,1,x1,mtype) * (P(2,2,x0,mtype)^2));
                fnj_other(x,2) = .5 * (P(1,2,x1,mtype) * P(2,2,x0,mtype)^3)...
                    + .5 * (P(1,2,x0,mtype) * P(2,2,x1,mtype) * (P(2,2,x0,mtype)^2));
                fnj_other(x,3) = .5 * (P(1,3,x1,mtype) * (P(2,2,x0,mtype)^3)...
                    + P(1,2,x1,mtype) * 3 * P(2,3,x0,mtype) * (P(2,2,x0,mtype)^2))...
                    + .5 * (P(1,3,x0,mtype) * P(2,2,x1,mtype) * (P(2,2,x0,mtype)^2)...
                    + P(1,2,x0,mtype) * (P(2,3,x1,mtype) * (P(2,2,x0,mtype)^2) + P(2,2,x1,mtype) * 2 * P(2,3,x0,mtype) * P(2,2,x0,mtype)));
                fnj_other(x,4) = .5 * (P(1,3,x1,mtype) * 3 * P(2,3,x0,mtype) * (P(2,2,x0,mtype)^2)...
                    + P(1,2,x1,mtype) * 3 * (P(2,3,x0,mtype)^2) * P(2,2,x0,mtype))...
                    + .5 * (P(1,3,x0,mtype) * P(2,3,x1,mtype) * (P(2,2,x0,mtype)^2)...
                    + P(1,3,x0,mtype) * P(2,2,x1,mtype) * 2 * P(2,3,x0,mtype) * P(2,2,x0,mtype)...
                    + P(1,2,x0,mtype) * P(2,3,x1,mtype) * 2 * P(2,3,x0,mtype) * P(2,2,x0,mtype)...
                    + P(1,2,x0,mtype) * P(2,2,x1,mtype) * (P(2,3,x0,mtype)^2));

        
                nj = 2;         % If today's # of rivals = 2
                x = 1 + (4*4*4)*ni + (4*4)*nj + 4*z1 + z2;  % index of today's own state
                x0 = 1 + (4*4*4)*0 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 0 shops
                x1 = 1 + (4*4*4)*1 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 1 shops
                x2 = 1 + (4*4*4)*2 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 2 shops
                x3 = 1 + (4*4*4)*3 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 3 shops
                
                fnj_mcd(x,1) = (P(2,1,x1,mtype)^2) * (P(2,2,x0,mtype)^2);
                fnj_mcd(x,2) = (1/3) * (P(2,1,x2,mtype) * (P(2,2,x0,mtype)^3))...
                    + (2/3) * (P(2,1,x1,mtype) * P(2,2,x1,mtype) * (P(2,2,x0,mtype)^2));
                fnj_mcd(x,3) = (1/3) * (P(2,2,x2,mtype) * (P(2,2,x0,mtype)^3))...
                    + (2/3) * ((P(2,2,x1,mtype)^2) * (P(2,2,x0,mtype)^2));
                fnj_mcd(x,4) = (1/3) * (P(2,3,x2,mtype) * (P(2,2,x0,mtype)^3)...
                    + (P(2,2,x2,mtype) * 3 * P(2,3,x0,mtype) * (P(2,2,x0,mtype)^2)));
                
                fnj_other(x,1) = .5 * 0 + .5 * (P(1,1,x1,mtype) * P(2,1,x1,mtype) * (P(2,2,x0,mtype)^2));
                fnj_other(x,2) = .5 * (P(1,1,x2,mtype) * (P(2,2,x0,mtype)^3))...
                    + .5 * (P(1,1,x1,mtype) * P(2,2,x1,mtype) * (P(2,2,x0,mtype)^2)...
                    + P(1,2,x1,mtype) * P(2,1,x1,mtype) * (P(2,2,x0,mtype)^2));
                fnj_other(x,3) = .5 * (P(1,2,x2,mtype) * (P(2,2,x0,mtype)^3))...
                    + .5 * (P(1,2,x1,mtype) * P(2,2,x1,mtype) * (P(2,2,x0,mtype)^2));
                fnj_other(x,4) = .5 * (P(1,3,x2,mtype) * (P(2,2,x0,mtype)^3)...
                    + P(1,2,x2,mtype) * 3 * P(2,3,x0,mtype) * P(2,2,x0,mtype))...
                    + .5 * (P(1,3,x1,mtype) * P(2,2,x1,mtype) * (P(2,2,x0,mtype)^2)...
                    + P(1,2,x1,mtype) * P(2,3,x1,mtype) * (P(2,2,x0,mtype)^2)...
                    + P(1,2,x1,mtype) * P(2,2,x1,mtype) * 2 * P(2,3,x0,mtype) * P(2,2,x0,mtype));

                nj = 3;         % If today's # of rivals = 3
                x = 1 + (4*4*4)*ni + (4*4)*nj + 4*z1 + z2;  % index of today's own state
                x0 = 1 + (4*4*4)*0 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 0 shops
                x1 = 1 + (4*4*4)*1 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 1 shops
                x2 = 1 + (4*4*4)*2 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 2 shops
                x3 = 1 + (4*4*4)*3 + (4*4)*ni + 4*z1 + z2;  % index of today's state for rivals with 3 shops
                
                fnj_mcd(x,1) = (2/3) * (P(2,1,x1,mtype)^3) * P(2,2,x0,mtype);
                fnj_mcd(x,2) = (2/3) * (P(2,1,x1,mtype)^2) * P(2,2,x1,mtype)...
                    + (1/3) * P(2,1,x2,mtype) * P(2,1,x1,mtype) * (P(2,2,x0,mtype)^2);
                fnj_mcd(x,3) = (2/3) * P(2,1,x1,mtype) * (P(2,2,x1,mtype)^2)...
                    + (1/3) * (P(2,1,x2,mtype) * P(2,2,x1,mtype) * (P(2,2,x0,mtype)^2)...
                    + P(2,2,x2,mtype) * P(2,1,x1,mtype) * (P(2,2,x0,mtype)^2));
                fnj_mcd(x,4) = (2/3) * (P(2,2,x1,mtype)^3) * P(2,2,x0,mtype)...
                    + (1/3) * P(2,2,x2,mtype) * P(2,2,x1,mtype) * (P(2,2,x0,mtype)^2);
                
                fnj_other(x,1) = .25 * (P(1,1,x1,mtype) * (P(2,1,x1,mtype)^2) * P(2,2,x0,mtype));
                fnj_other(x,2) = .25 * 0 + .25 * (P(1,1,x2,mtype) * P(2,1,x1,mtype) * (P(2,2,x0,mtype)^2))...
                    + .25 * (P(1,1,x1,mtype) * 2 * P(2,1,x1,mtype) * P(2,2,x1,mtype) * P(2,2,x0,mtype)...
                    + P(1,2,x1,mtype) * (P(2,1,x1,mtype)^2) * P(2,2,x0,mtype))...
                    + .25 * (P(1,1,x1,mtype) * P(2,1,x2,mtype) * (P(2,2,x0,mtype)^2));
                fnj_other(x,3) = .25 * (P(1,1,x3,mtype) * (P(2,2,x0,mtype)^3))...
                    + .25 * (P(1,1,x2,mtype) * P(2,2,x1,mtype) * (P(2,2,x0,mtype)^2)...
                    + P(1,2,x2,mtype) * P(2,1,x1,mtype) * (P(2,2,x0,mtype)^2))...
                    + .25 * (P(1,1,x1,mtype) * (P(2,2,x1,mtype)^2) * P(2,2,x0,mtype)...
                    + P(1,2,x1,mtype) * 2 * P(2,1,x1,mtype) * P(2,2,x1,mtype) * P(2,2,x0,mtype))...
                    + .25 * (P(1,1,x1,mtype) * P(2,2,x2,mtype) * (P(2,2,x0,mtype)^2)...
                    + P(1,2,x1,mtype) * P(2,1,x2,mtype) * (P(2,2,x0,mtype)^2));
                fnj_other(x,4) = .25 * (P(1,2,x3,mtype) * (P(2,2,x0,mtype)^3))...
                    + .25 * (P(1,2,x2,mtype) * P(2,2,x1,mtype) * (P(2,2,x0,mtype)^2))...
                    + .25 * (P(1,2,x1,mtype) * (P(2,2,x1,mtype)^2) * P(2,2,x0,mtype))...
                    + .25 * (P(1,2,x1,mtype) * P(2,2,x2,mtype) * (P(2,2,x0,mtype)^2));
                
            end
        end
    end
    
    Denom_mcd = sum(fnj_mcd,2) * ones(1,4);         % Sum of each row
    fnj_mcd = fnj_mcd ./ Denom_mcd;                 % Make sure each row sums up to 1
    fnj_mcd(isnan(fnj_mcd)) = 0;                    % Replace NaN with 0
    F_Nj(1,j,:,:,mtype) = ...
        [fnj_mcd(:,1) * ones(1,16), fnj_mcd(:,2) * ones(1,16), fnj_mcd(:,3) * ones(1,16), fnj_mcd(:,4) * ones(1,16),...
         fnj_mcd(:,1) * ones(1,16), fnj_mcd(:,2) * ones(1,16), fnj_mcd(:,3) * ones(1,16), fnj_mcd(:,4) * ones(1,16),...
         fnj_mcd(:,1) * ones(1,16), fnj_mcd(:,2) * ones(1,16), fnj_mcd(:,3) * ones(1,16), fnj_mcd(:,4) * ones(1,16),...
         fnj_mcd(:,1) * ones(1,16), fnj_mcd(:,2) * ones(1,16), fnj_mcd(:,3) * ones(1,16), fnj_mcd(:,4) * ones(1,16)];

    Denom_other = sum(fnj_other,2) * ones(1,4);             % Sum of each row
    fnj_other = fnj_other ./ Denom_other;                   % Make sure each row sums up to 1
    fnj_other(isnan(fnj_other)) = 0;                        % Replace NaN with 0
    F_Nj(2,j,:,:,mtype) = ...
        [fnj_other(:,1) * ones(1,16), fnj_other(:,2) * ones(1,16), fnj_other(:,3) * ones(1,16), fnj_other(:,4) * ones(1,16),...
         fnj_other(:,1) * ones(1,16), fnj_other(:,2) * ones(1,16), fnj_other(:,3) * ones(1,16), fnj_other(:,4) * ones(1,16),...
         fnj_other(:,1) * ones(1,16), fnj_other(:,2) * ones(1,16), fnj_other(:,3) * ones(1,16), fnj_other(:,4) * ones(1,16),...
         fnj_other(:,1) * ones(1,16), fnj_other(:,2) * ones(1,16), fnj_other(:,3) * ones(1,16), fnj_other(:,4) * ones(1,16)];
    F_Nj(3:5,j,:,:,mtype) = repmat(F_Nj(2,j,:,:,mtype), [3, 1, 1, 1, 1]);    % Other 4 chains are symmetric
    
end

for j = 2:J    
    F_Nj(:,j,:,:,:) = F_Nj(:,1,:,:,:);    
end

F = F_Ni .* F_Nj .* F_dz1 .* F_dz2;
